{% extends "base.html" %}

{% block content %}
	<h1>{{ competition }}</h1>
	<h2>{{ season }}</h2>
	<h3>{{ division }}</h3>
	{% if divisiongroup %}
		<h4>{{ divisiongroup }}</h4>
	{% endif %}

	<h3>Draw</h3>

	<p><em>render the list of matches here</em></p>
{% endblock %}
