# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-07-04 22:02
from __future__ import unicode_literals

import django.db.models.deletion
import touchtechnology.common.db.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('competition', '0029_seasonreferee'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='seasonreferee',
            options={'ordering': ('club', 'season', 'person__last_name', 'person__first_name'), 'verbose_name': 'referee'},
        ),
        migrations.AddField(
            model_name='match',
            name='referees',
            field=touchtechnology.common.db.models.ManyToManyField(blank=True, related_name='matches', to='competition.SeasonReferee'),
        ),
    ]
