{% load i18n %}
<div class="col-lg-4 col-md-4 col-sm-4">
	<div class="portlet portlet-table">
		<div class="portlet-header">
			<h4 class="portlet-title">{{ widget.verbose_name }}</h4>
		</div>
		<div class="portlet-body">
			<table class="table table-striped table-condensed">
				<thead>
					<tr>
						<th>{% trans "Division" %}</th>
						<th>{% trans "Stage" %}</th>
					</tr>
				</thead>
				<tbody>
					{% for division, object_list in stages.items %}
						<tr>
							<td>{{ division.title }}</td>
							<td>
								<ul>
									{% for stage in object_list %}
										{% with competition=stage.division.season.competition season=stage.division.season division=stage.division %}
											<li><a href="{% url 'admin:fixja:competition:season:division:stage:draw:progress' competition.pk season.pk division.pk stage.pk %}">{{ stage.title }}</a></li>
										{% endwith %}
									{% endfor %}
								</ul>
							</td>
						</tr>
					{% empty %}
						<tr>
							<td colspan="2">{% trans "No stages require progression." %}</td>
						</tr>
					{% endfor %}
				</tbody>
			</table>
		</div>
	</div>
</div>
