.table tbody>tr>td{vertical-align:middle}.table tbody>tr>td .fa-ul{margin-bottom:0}ol.breadcrumb:empty{display:none}.form-group{margin-bottom:15px}.form-group.split_array_widget .form-control{margin-bottom:.4em}.form-group.select_date_widget div select,.form-group.select_date_time_widget div select{display:inline-block}.form-group.select_date_widget div select:nth-child(1),.form-group.select_date_time_widget div select:nth-child(1){width:24%}.form-group.select_date_widget div select:nth-child(2),.form-group.select_date_time_widget div select:nth-child(2){width:49%}.form-group.select_date_widget div input,.form-group.select_date_time_widget div input{display:inline-block;width:24%}.form-group.select_date_time_widget .time_part{display:block;margin-top:.4em}.form-group.select_date_time_widget .time_part select:nth-child(1){width:24%}.form-group.select_date_time_widget .time_part select:nth-child(2){width:24%}.form-group.select_date_time_widget .time_part select:nth-child(3){width:49%}.form-group .icheck{max-height:15em;overflow-y:scroll}.form-group .icheck .ui-icheck .form-control{height:auto}.help-block{margin-bottom:5px}.select2-search-choice{width:100%}