{% extends "mvp/list.inc.html" %}

{% block row-buttons %}
	<div class="btn-group">
		{% block row-buttons-extra %}{% endblock %}
		<button type="button" class="btn btn-default btn-sm dropdown-toggle" data-toggle="dropdown" aria-expanded="false">
			<span class="caret"></span>
		</button>
		<ul class="dropdown-menu dropdown-menu-right" role="menu">
			{% block row-buttons-items %}
				{{ block.super }}
			{% endblock %}
		</ul>
	</div>
{% endblock %}
