{% extends "mvp/list.inc.html" %}
{% load i18n %}

{% block table-headings %}
	{{ block.super }}
	<th>{% trans "Teams" %}</th>
	<th>{% trans "Finals" %}</th>
{% endblock table-headings %}

{% block table-columns %}
	{{ block.super }}
	<td>{{ obj.teams|default:"-" }}</td>
	<td>
		{% if obj.is_final %}
			<i class="fa fa-trophy"></i>
		{% endif %}
	</td>
{% endblock table-columns %}