{% load i18n %}
{% load common %}
<td class="team {{ opponent.club.slug|cssify }}">
	{% if opponent.slug %}
		{% url application.name|add:":team" competition=competition.slug season=season.slug division=division.slug team=opponent.slug as url1 %}
		{% url application.name|add:":team" season=season.slug division=division.slug team=opponent.slug as url2 %}
		{% url application.name|add:":team" division=division.slug team=opponent.slug as url3 %}
		<a href="{{ url1|default:url2|default:url3 }}">{% trans opponent.title %}</a>
	{% else %}
		{% trans opponent.title %}
	{% endif %}
</td>
