<html>{% load common %}
<head>
	<title>querystring test</title>
</head>
<body>
	{% for object in object_list %}
		{% spaceless %}
			<p>
				<span id="{{ object.pk }}">{{ object.pk }}</span>
				<em>{{ object.datetime|timesince }}</em>
			</p>
		{% endspaceless %}
	{% endfor %}

	{% pagination %}
</body>
</html>
